function hapdata

%Code copyright 2002 by M. B. Hamilton, J. R. Miller and Georgetown University.
%Code may be modified as long as credit is given to the original authors.

%function to estimate the ANOVA-based estimate of population structure theta (see Weir)
%for an actual haploid data set.  Includes confidence interval estimation by bootstrapping
%over populations.

%variables for resampling
bootreps = 5000;    %number of bootstrap replicates, must use at least one
sigalpha = 0.05;    %significance level for confidence interval, provides (1-sigalpha)% CI

%organelle genotype matrix has one row for each genotytpe (A) per locus
%each column is one population in both cases

%orggen contains our data as haplotype counts, this is entered by the user
%each row contains the counts of one haplotype
%each column contains the counts for all haplotypes in one population
%for example, orggen = [10 5 0; 0 5 10]; says that there are three pops,
%ten individuals per pop, and pop one contains 10 A haplotypes and 0 a
%haplotypes.

%All haplotypes used in the matrix below must have data for ALL indels/bases/cut sites 
%(or whatever) used to define haplotypes!!!  Missing data is not allowed as it would be
%for nuclear loci.

%In the case below there are 8 populations (columns) and 4 haplotypes (rows).
%20 individuals have been sampled in each population.  REPLACE THIS WITH YOUR DATA.

orggen = [20 10  0   0 16 20  0  0; 
           0  0 20   5  0  0 20  0; 
           0  0  0  15  0  0  0  1;
           0 10  0   0  4  0  0 19];

npops = size(orggen,2); %number of populations, same as number of columns in orggen
nhaplotypes = size(orggen,1); %number of haplotypes, same as number of rows in orggen
N = sum(orggen,1); %row vector of population sizes, each pop can have a unique number of individuals

%do anova haploid
    %get point estimate first
    pops = (1:npops);     %vector of all populations in sample
    [thetahap, cthetahap] = anovahap(orggen,npops,N,nhaplotypes,pops);
    
    %then get POPULATION resampled estimates
	for y=1:bootreps
        rpops = resamplepops(npops,pops); %sample with replacement from all populations in sample
        [thetahapreps(:,y),cthetahapreps(y)]=anovahap(orggen,npops,N,nhaplotypes,rpops);
    end
    [haplowbds, hapupbds] = bootconfint(thetahapreps,sigalpha,bootreps);
    [chaplowbds, chapupbds] = bootconfint(cthetahapreps,sigalpha,bootreps);
    
%politely output our results
disp('Overall estimate of theta (averaged over all haplotypes):');
disp(cthetahap);
disp('The POPULATION BOOTSTRAP confidence interval for the overall estimate of theta:');
disp(chaplowbds);
disp(chapupbds);

disp('The confidence interval is')
ci=(1-sigalpha)*100;
disp(ci);
disp('percent.');
disp('note: BOOTSTRAPING OVER LOCI IS INAPPLICABLE UNLESS THERE IS RECOMBINATION');
disp('Boostrap CI based on');
disp(bootreps);
disp('replicates.');
    
%_______________________________________
function [thetahap, cthetahap]=anovahap(orggen,npops,N,nhaplotypes,pops)
%ANOVA estimate for haploids, follows Weir pp. 171-173

%inputs:
%orggen: contains haplotype count data
%npops: number of populations in sample
%N: row vector of number of individuals in each population
%nhaplotypes: number of observed haplotypes in data
%pops: populations sampled

%output:
%thetahat: column vector containing estimate for each haplotype
%cthetahat: scalar estimate of theta for combined haplotypes
N = N(pops);
Nmat = repmat(N, nhaplotypes, 1); %make full matrix of pop sizes
orgfreq=orggen(1:nhaplotypes,pops)./Nmat; %get frequency of each haplotype for sampled pops

%Weir p. 171
dfwithin = Nmat-1;
sumdfwithin = sum(dfwithin,2);
msg=(1./sumdfwithin).*sum(Nmat.*orgfreq.*(1-orgfreq),2);
%msg=1/(npops*(N-1))*sum(N*orgfreq.*(1-orgfreq),2);

msp=1/(npops-1)*sum(Nmat.*((orgfreq-mean(orgfreq,2)*ones(1,npops)).^2),2);

%Weir p. 172
sqNimat = Nmat.^2;
sumsqNimat = sum(sqNimat,2);
sumNimat = sum(Nmat, 2);
ratio = sumsqNimat./sumNimat;
sumnidev = sum(Nmat,2)-ratio;
nc=1/(npops-1).*sumnidev;

%nc=1/(npops-1)*(npops*N-(npops*N^2)/(npops*N));

%Weir p.l 173
numerator=msp-msg;
denominator=msp+(nc-1).*msg;

for i=1:nhaplotypes
    if denominator(i)==0
        thetahap(i,1)=0.0;
    else
        thetahap(i,1)=numerator(i)/denominator(i);
    end
end
%thetahat

cnumerator=sum(numerator,1);
cdenominator=sum(denominator,1);
if cdenominator==0
        cthetahap=0.0;
    else
        cthetahap=cnumerator/cdenominator;
    end

%_______________________________________
function [rpops] = resamplepops(npops,pops)
%resample populations with replacement for bootstrap
%test to see if resampled pops are all the same population,
%if so resample again

%inputs:
%npops = the number of populations sampled
%pops = the row vector of indices of sampled populations

    dup=1;
    while dup==1
		index = unidrnd(npops,1,npops); % get random set of npops indices
		rpops = pops(1,index); % assign pops based on indices, or sample with replacement from pops
        mx=max(rpops);   %get first index of maximum value
        i=find(rpops==mx);   %get all other indices that are equal to maximum value
        numdups = length(i);   %get size of array of indices that are equal to max
        if numdups==npops
            dup=1;  %all pops are the same
        else
            dup=0;  %all pops are not the same
        end
    end

%_______________________________________
function [bootlowbds, bootupbds]=bootconfint(thetareps, alpha, numreps)
% makes bootstrap confidence interval
% inputs:
% fname = name of estimator function (character string; put it in quotes)
% thetareps = vector or array of point estimates from bootstrap procedure
% dimensions: (locus, replicate estimate)
% (may be output of bootdip.m or boothap.m)
% alpha: request (1-alpha) confidence interval
% outputs:
% [bootlowbds, bootupbds] = lower and upper bounds on confidence interval
% dimensions: (locus) 
% note: recall that BOOTSTRAP OVER LOCI IS INAPPLICABLE FOR HAPLOIDS

%numreps=length(thetareps);
% length of confindence interval
intlength = min(round((1-alpha)*numreps),numreps);

% find best confidence interval
% note: for now, we find the SHORTEST INTERVAL in absolute terms.
numloci=size(thetareps);
numloci=numloci(1);

for locus=1:numloci % find CI for each locus separately
thetareps(locus,:)=sort(thetareps(locus,:));
bootlowbd=1;
bootupbd=intlength;
confwidth=thetareps(locus,bootupbd)-thetareps(locus,bootlowbd);
for i=2:numreps-intlength+1
    testwidth=thetareps(locus,i+intlength-1)-thetareps(locus,i);
    if testwidth<confwidth
        confwidth=testwidth;
        bootupbd=i+intlength-1;
        bootlowbd=i;
    end
end

% convert indices to confidence limits
bootlowbds(locus)=thetareps(locus,bootlowbd);
bootupbds(locus)=thetareps(locus,bootupbd);
end

% make column vectors
bootlowbds=bootlowbds.';
bootupbds=bootupbds.';  
%_______________________________________
